module Lexer(lexer,isWhite,combineText,isTSpace) where
import AST
import Array
import PackedString

lexer :: String -> [Token]
lexer []    =  [] 
lexer (c:cs)
  | isWhite c  = lexSpace (c:cs) 
  | c == '<'   = lexTag cs
  | otherwise  = lexWord (c:cs) 

lexSpace (c:cs) = lexSpc cs ""
      where lexSpc []     acc = [tSpace (c:reverse acc)]
            lexSpc (a:as) acc 
	     | isWhite a      = lexSpc as (a:acc)
	     | True           = tSpace (c:reverse acc) : lexer (a:as)

lexWord (c:cs) = lexWd cs ""
      where lexWd []     acc = [tText (c:reverse acc)]
            lexWd (a:as) acc 
	     | not (isWhite a) && a /= '<' = lexWd as (a:acc)
	     | True                        = tText (c:reverse acc) : lexer (a:as)
{-
lexSpace (c:cs)= let (w,rest) = span isWhite cs
		 in TSpace (c:w): lexer rest

lexWord (c:cs) = let (w,rest) = span (\c -> (not . isWhite) c && c /= '<') cs
                 in TText (c:w):lexer rest 
-}
lexTag ('/':c:cs) = EndTag (tagtable ! c):lexer (tail (dropWhile (/='>') cs))
lexTag cs = case (span (/='>') cs) of
                (c:_, _:rest) -> TTag (tagtable ! c):lexer rest

tagtable :: Array Char Tag
tagtable = array ('0','y') [ ('B',B), ('E',EM), ('I',I), ('P',PL), ('S',S), ('T',TT)
  , ('U',U), ('0',Size 0), ('1',Size 1), ('2',Size 2), ('3',Size 3)
  , ('4', Size 4), ('5',Size 5), ('6',Size 6), ('7',Size 7)
  , ('8',Size 8), ('9',Size 9), ('r',Color Red), ('g',Color Green)
  , ('b',Color Blue), ('c',Color Cyan), ('m',Color Magenta), ('y',Color Yellow)
  , ('k',Color Black), ('w',Color White)
  ]
                         
           
isWhite ' '    = True
isWhite '\x0d' = True
isWhite '\x0a' = True
isWhite '\x09' = True
isWhite _      = False

-- isWhite = (`elem`" \x0d\x0a\x09")

-- This is useful to call after redundant spaces have been removed.
combineText :: [Token] -> [Token]
combineText [] = []
combineText ts =
  case span isText ts of
    ([],t:ts) -> t:combineText ts
    (ts1,ts2) -> if all isTSpace ts1
		 then TSpace (join ts1):combineText ts2
		 else TText  (join ts1):combineText ts2
  where
    join = concatPS . map getText

    isText (TText  _) = True
    isText (TSpace _) = True
    isText _          = False

    getText (TText s) = s
    getText (TSpace s) = s

isTSpace (TSpace _) = True
isTSpace _          = False
