module Meaning(Meaning,meaning,start) where
import Context
import AST
import Lexer(isWhite)
import PackedString

default(Int) -- just to help Hugs get the types right

type Meaning = [(Char,PropertyRecord)]
type ExtMeaning = [(Str,ContextStack)]
type ContextStack = (PropertyRecord,(Int,[Tag]))
startContext = (rootContext,(0,[]))

meaning :: [Token] -> Meaning
meaning ts = [(c,p)|(s,(p,_))<-extmeaning ts,c<-unpackPS s]
{-
rmspace :: [Token] -> [Token]
rmspace = restoreTags (snd startContext) . extmeaning
  where
    restoreTags tgs ts =
      case ts of
        [] -> map endtag (snd tgs)
	(s,(_,tgs')):ts -> tags tgs tgs'++txt s:restoreTags tgs' ts

    txt s = if isW s then TSpace s else TText s
    isW (c:_) = isWhite c

    tags (n,tgs) (n',tgs') = pop++pop2++push2++push
      where
        m = min n n'
        pop = map endtag (take (n-m) tgs)
	push = reverse (map TTag (take (n'-m) tgs'))
	pop2 = reverse (map endtag old)
	push2 = map TTag new
	(old,new) = unzip $ dropWhile (uncurry (==)) bot
        bot = reverse (zip (drop (n-m) tgs) (drop (n'-m) tgs'))
-}
--endtag t = EndTag -- hmm, no good for pretty printing
endtag t = EndTag t

extmeaning :: [Token] -> ExtMeaning
extmeaning = cleanSpace . addContext [startContext]

cleanSpace :: [(Token,ContextStack)] -> ExtMeaning
cleanSpace ts =
  case ts of
    [] -> []
    (TText s,p):ts -> (s,p):cleanSpace ts
    (TSpace s,p):ts | tt (fst p) -> (s,p):cleanSpace ts
    t1@(TSpace s1,p1):(TSpace _,p2):ts | fst p1 == fst p2 -> cleanSpace (t1:ts)
    (TSpace s,p):ts -> (sp,p):cleanSpace ts

sp = packString " "

addContext :: [ContextStack] -> [Token] -> [(Token,ContextStack)]
addContext p [] = []
addContext ps@(p:ps') (t:ts) =
  case t of
    --TText "" -> addContext ps ts
    TText s -> (t,p):addContext ps ts
    --TSpace "" -> addContext ps ts
    TSpace s -> (t,apFst spaceContext p):addContext ps ts
    TTag t -> addContext (push t p:ps) ts
    EndTag _ -> addContext ps' ts

push t (p,(n,ts)) = (start t p,(n+1,t:ts))

start t p =
  case t of
    B       -> p{b=True}
    EM      -> if s p then p else p{em=not (em p)}
    I       -> p{i=True}
    PL      -> plainContext p
    S       -> p{s=True,em=False}
    TT      -> p{tt=True}
    U       -> if u0<3 then p{u=u0+1} else p
        where u0 = u p
    Size s  -> p{size=s}
    Color c -> p{color=c}

------

apFst f (x,y) = (f x,y)
