module PPrint(ppDoc,ppTag,ppTokens,ppToken) where

import AST
import PackedString


ppDoc :: Doc -> String
ppADoc :: ADoc -> String

-- New, efficient implementation of ppDoc:
ppDoc ds = ppDoc' ds ""
ppADoc d = ppADoc' d "" -- probably not useful

ppDoc' :: Doc -> ShowS
ppDoc' ds rest = foldr ppADoc' rest ds

ppADoc' :: ADoc -> ShowS
ppADoc' (Text s)  = showPString s
ppADoc' (Tag t d) = showString (ppTag t) .
		    ppDoc' d .
		    showString (ppEndTag t)
ppADoc' (Space s) = showPString s

{- Old version of ppDoc:
ppDoc ds = foldr (\x s -> ppADoc x ++ s) "" ds

ppADoc (Text s)  = s
ppADoc (Tag t d) = let pt = ppTag t
                   in "<" ++ pt ++ ">" ++ (ppDoc d ++ ("</" ++ pt ++ ">"))
ppADoc (Space s) = s
-}

ppTag :: Tag -> String
ppTag B  = "<B>"
ppTag EM = "<EM>"
ppTag I  = "<I>"
ppTag PL = "<PL>"
ppTag S  = "<S>"
ppTag TT = "<TT>"
ppTag U  = "<U>"
ppTag (Size i) = "<" ++ show i ++ ">"
ppTag (Color c) = "<" ++ ppCol c ++ ">"
ppTag (STag ts) = concat $ map ppTag ts

ppEndTag :: Tag -> String
ppEndTag B  = "</B>"
ppEndTag EM = "</EM>"
ppEndTag I  = "</I>"
ppEndTag PL = "</PL>"
ppEndTag S  = "</S>"
ppEndTag TT = "</TT>"
ppEndTag U  = "</U>"
ppEndTag (Size i) = "</" ++ show i ++ ">"
ppEndTag (Color c) = "</" ++ ppCol c ++ ">"
ppEndTag (STag ts) = concat $ map ppEndTag $ reverse ts


ppCol Red     = "r"
ppCol Green   = "g"
ppCol Blue    = "b"
ppCol Cyan    = "c"
ppCol Magenta = "m"
ppCol Yellow  = "y"
ppCol Black   = "k"
ppCol White   = "w"

ppTokens :: [Token] -> String
ppTokens = concatMap ppToken

ppToken :: Token -> String
ppToken t =
  case t of
    TText s -> unpackPS s
    TTag t -> ppTag t
    EndTag t -> ppEndTag t
    TSpace s -> unpackPS s


showPString = showString . unpackPS
