module PushSpaces (pushSpaces) where
import AST
import PackedString

pushSpaces :: [ADoc] -> [ADoc]
pushSpaces = pushsp (False,False)

pushsp :: (Bool,Bool) -> [ADoc] -> [ADoc]
pushsp st@(ulined,teed) [] = []
pushsp st@(ulined,teed) (d:ds) =
  case (d,ds) of
    (Text s1, (Text  s2:ds')) -> pushsp st (Text (s1 `appendPS` s2) : ds')
    (Text s1, (Space s2:ds')) -> pushsp st (Text (s1 `appendPS` s2) : ds')
    (Space s, (Tag t ks:ds')) -> if acceptsSpace t
                                   then pushsp st (Tag t ([Space s]+++ks) : ds')
                                   else d : pushsp st ds
    (Tag t ks, (Space s:ds')) -> if acceptsSpace t
                                   then pushsp st (Tag t (ks+++[Space s]) : ds')
                                   else Tag t (recurseOn t ks) : pushsp st ds
    (Tag t ks, _) -> Tag t (recurseOn t ks) : pushsp st ds
    _             -> d : pushsp st ds
  
  where acceptsSpace (Color _) = not ulined
        acceptsSpace (Size _)  = False
        acceptsSpace PL        = not ulined && not teed
        acceptsSpace TT        = teed
        acceptsSpace U         = False
        acceptsSpace _         = True
        recurseOn PL ks = pushsp (False,False) ks
        recurseOn TT ks = pushsp (ulined,True) ks
        recurseOn U ks = pushsp (True,teed) ks
        recurseOn _ ks = pushsp st ks
       
 
(+++) :: [ADoc] -> [ADoc] -> [ADoc]
infixr 5 +++

ds1 +++ ds2@((Tag _ _):_) = ds1 ++ ds2
[] +++ ds = ds
[d1] +++ (d2:ds) =
  case (d1,d2) of
    (Text  s1, Text  s2) -> (Text (s1 `appendPS`s2)) : ds
    (Space s1, Text  s2) -> (Text (s1 `appendPS`s2)) : ds
    (Text  s1, Space s2) -> (Text (s1 `appendPS`s2)) : ds
    (Space s1, Space s2) -> (Space (s1 `appendPS`s2)) : ds
    _                    -> d1:d2:ds
(d:ds) +++ ds2 = d : (ds +++ ds2)
