module SizeColor (optColors, optSizes) where
import AST
import List (findIndices)
import Parser
import PackedString

getDocColor :: ADoc -> Maybe (Tag, Doc)
getDocColor (Tag t@(Color _) ks) = Just (t, ks)
getDocColor (Tag t [d]) = case getDocColor d of
                             Nothing -> Nothing
                             Just (c,ts) -> Just (c,[Tag t ts])
getDocColor _ = Nothing

getDocSize :: ADoc -> Maybe (Tag, Doc)
getDocSize (Tag t@(Size _) ks) = Just (t, ks)
getDocSize (Tag t [d]) = case getDocSize d of
                             Nothing -> Nothing
                             Just (z,ts) -> Just (z,[Tag t ts])
getDocSize _ = Nothing

bestPair allAttrs attrList = 
  if null pairs then Nothing else Just (minimumOf (countCutsWith pairs) pairs)
  where
    pairs = concatMap (mkPairs.(\n -> findIndices (n==) attrList)) allAttrs

mkPairs (x1:xs@(x2:_)) = (x1,x2):mkPairs xs
mkPairs _ = []
countCutsWith xs x = length . filter (cuts x) $ xs
cuts (a,b) (c,d) = (c<b) && (a<d) && ((a<c)==(b<d))
minimumOf f = foldl1 (minOf f)
minOf f x y | f x <= f y = x
            | otherwise  = y


optimizeAttrs :: ([Tag],ADoc -> Maybe (Tag, Doc)) -> [ADoc] -> [ADoc]
optimizeAttrs _ [] = []
optimizeAttrs ps@(allAttrs,getAttr) (d:ds) =
  case bestPair allAttrs attrList of
    Nothing    -> recurseOn d : optimizeAttrs ps ds
    Just (a,b) -> let (left, l:xs) = splitAt a (d:ds)
                      (m, r:right) = splitAt (b-a-1) xs
                      Just (t',l') = getAttr l
                      Just (_, r') = getAttr r
                      mid = Tag t' (l' ++ m ++ r')
                  in optimizeAttrs ps (left ++ mid : right)
  where 
    attrList = map fst (takeJusts (map getAttr (d:ds)))
    recurseOn (Tag t xs) = Tag t (optimizeAttrs ps xs)
    recurseOn x = x
  
takeJusts :: [Maybe a] -> [a]
takeJusts [] = []
takeJusts (Nothing:_) = []
takeJusts (Just x:xs) = x : takeJusts xs

optColors =
  optimizeAttrs
    (map Color [Red, Green, Blue, Cyan, Magenta, Yellow, Black, White], getDocColor)

optSizes = optimizeAttrs (map Size [0..9], getDocSize)

test = "<r>a</r><g>a</g><b>a</b><g>a</g><r>a</r>"
